function [features] = extractFeaturesFromHsIntervals(assigned_states, PCG)
%
% This function calculate 20 features based on the assigned_states by running "runSpringerSegmentationAlgorithm.m" function
%
% INPUTS:
% assigned_states: the array of state values assigned to the sound recording.
% PCG: resampled sound recording with 1000 Hz.
%
% OUTPUTS:
% features: the obtained 20 features for the current sound recording
%
%
% Written by: Chengyu Liu, January 22 2016
%             chengyu.liu@emory.edu
%
% Last modified by:
%
%
% $$$$$$ IMPORTANT
% Please note: the calculated 20 features are only some pilot features, some features maybe
% helpful for classifying normal/abnormal heart sounds, some maybe
% not. You need re-construct the features for a more accurate classification.

% 

%% We just assume that the assigned_states cover at least 2 whole heart beat cycle
indx = find(abs(diff(assigned_states))>0); % find the locations with changed states

if assigned_states(1)>0   % for some recordings, there are state zeros at the beginning of assigned_states
    switch assigned_states(1)
        case 4
            K=1;
        case 3
            K=2;
        case 2
            K=3;
        case 1
            K=4;
    end
else
    switch assigned_states(indx(1)+1)
        case 4
            K=1;
        case 3
            K=2;
        case 2
            K=3;
        case 1
            K=0;
    end
    K=K+1;
end

indx2                = indx(K:end);
rem                  = mod(length(indx2),4);
indx2(end-rem+1:end) = [];
A                    = reshape(indx2,4,length(indx2)/4)'; % A is N*4 matrix, the 4 columns save the beginnings of S1, systole, S2 and diastole in the same heart cycle respectively


%% Feature calculation
m_RR        = round(mean(diff(A(:,1))));             % mean value of RR intervals
sd_RR       = round(std(diff(A(:,1))));              % standard deviation (SD) value of RR intervals
mean_IntS1  = round(mean(A(:,2)-A(:,1)));            % mean value of S1 intervals
sd_IntS1    = round(std(A(:,2)-A(:,1)));             % SD value of S1 intervals
mean_IntS2  = round(mean(A(:,4)-A(:,3)));            % mean value of S2 intervals
sd_IntS2    = round(std(A(:,4)-A(:,3)));             % SD value of S2 intervals
mean_IntSys = round(mean(A(:,3)-A(:,2)));            % mean value of systole intervals
sd_IntSys   = round(std(A(:,3)-A(:,2)));             % SD value of systole intervals
mean_IntDia = round(mean(A(2:end,1)-A(1:end-1,4)));  % mean value of diastole intervals
sd_IntDia   = round(std(A(2:end,1)-A(1:end-1,4)));   % SD value of diastole intervals

%% Calculating ratio features for each cycle
for i=1:size(A,1)-1
    R_SysRR(i)  = (A(i,3)-A(i,2))/(A(i+1,1)-A(i,1))*100;
    R_DiaRR(i)  = (A(i+1,1)-A(i,4))/(A(i+1,1)-A(i,1))*100;
    R_SysDia(i) = R_SysRR(i)/R_DiaRR(i)*100;
    
    P_S1(i)     = sum(abs(PCG(A(i,1):A(i,2))))/(A(i,2)-A(i,1));
    P_Sys(i)    = sum(abs(PCG(A(i,2):A(i,3))))/(A(i,3)-A(i,2));
    P_S2(i)     = sum(abs(PCG(A(i,3):A(i,4))))/(A(i,4)-A(i,3));
    P_Dia(i)    = sum(abs(PCG(A(i,4):A(i+1,1))))/(A(i+1,1)-A(i,4));
    if P_S1(i)>0
        P_SysS1(i) = P_Sys(i)/P_S1(i)*100;
    else
        P_SysS1(i) = 0;
    end
    if P_S2(i)>0
        P_DiaS2(i) = P_Dia(i)/P_S2(i)*100;
    else
        P_DiaS2(i) = 0;
    end
end

% Finding mean and std for all the features
m_Ratio_SysRR   = mean(R_SysRR);  % mean value of the interval ratios between systole and RR in each heart beat
sd_Ratio_SysRR  = std(R_SysRR);   % SD value of the interval ratios between systole and RR in each heart beat
m_Ratio_DiaRR   = mean(R_DiaRR);  % mean value of the interval ratios between diastole and RR in each heart beat
sd_Ratio_DiaRR  = std(R_DiaRR);   % SD value of the interval ratios between diastole and RR in each heart beat
m_Ratio_SysDia  = mean(R_SysDia); % mean value of the interval ratios between systole and diastole in each heart beat
sd_Ratio_SysDia = std(R_SysDia);  % SD value of the interval ratios between systole and diastole in each heart beat

indx_sys = find(P_SysS1>0 & P_SysS1<100);   % avoid the flat line signal
if length(indx_sys)>1
    m_Amp_SysS1  = mean(P_SysS1(indx_sys)); % mean value of the mean absolute amplitude ratios between systole period and S1 period in each heart beat
    sd_Amp_SysS1 = std(P_SysS1(indx_sys));  % SD value of the mean absolute amplitude ratios between systole period and S1 period in each heart beat
else
    m_Amp_SysS1  = 0;
    sd_Amp_SysS1 = 0;
end
indx_dia = find(P_DiaS2>0 & P_DiaS2<100);
if length(indx_dia)>1
    m_Amp_DiaS2  = mean(P_DiaS2(indx_dia)); % mean value of the mean absolute amplitude ratios between diastole period and S2 period in each heart beat
    sd_Amp_DiaS2 = std(P_DiaS2(indx_dia));  % SD value of the mean absolute amplitude ratios between diastole period and S2 period in each heart beat
else
    m_Amp_DiaS2  = 0;
    sd_Amp_DiaS2 = 0;
end

%% features provided by Rohan

%[pxx,f] = pyulear(___,fs) autoregressive PSD

%% Spectram of diastole Schmidt method
[row, col] = size (A);

fs = 1000;
% define alpha and beta
alpha = 0.7;
beta = 0.7;

pm1 = [];
pm2 = [];
pm3 = [];
pm4 = [];
pa1 = [];
pa2 = [];
pa3 = [];
pa4 = [];
% p1 = [];
% p2 = [];
% p3 = [];

for i = 1:row-1
    temp_t = A(i,4):A(i+1,1);
    temp_amp = PCG(temp_t);
    
    % break the entire signal into 50ms non overlapping segments
    
    len = fs*(50/1000);
    num_block = floor(length(temp_amp)/len);
    
    % 5ms moving average window length
    % ma_len = (fs*5)/1000;
    
    abs_z = [];
    ang_z = [];
    % process every sub segment
    for j = 1:num_block
        %         % stationary part check
        x = temp_amp((j-1)*len+1:(j-1)*len+len);
        %         std_x = std(x);
        %         % calculate instantaneous variance (IVar)
        %         ivar = zeros(1,len-ma_len);
        %         for n = 1:(len-ma_len)
        %         ivar(n) = sum(x(n:n+ma_len-1).^2/std_x)/ma_len;
        %         end
        %         var_ivar = var(ivar);
        %         if (var_ivar>alpha)
        %             continue
        %         end
        
        % AR model of the subsegment
        a = arburg(x, 10);
        if(length(find(isnan(a)==1))>0)
            continue;
        end
        % poles of auto regressive model coefficients
        z = roots(a);
        z = z.';
        abs_z = [abs_z; abs(z)];
        ang_z = [ang_z;angle(z)];
    end
    
    if (isempty(abs_z))
        continue;
    end
    
%     % spectrum analysis based on welch algorithm
%     
%     [P, F] = pwelch(temp_amp, len, 0.9*len, 512, 1000);
%     
%     if (sum(P)==0)
%         continue;
%     end
    
    % mean pole magnitude of poles of all sub segment
    pm1 = [pm1 mean(abs_z(:,1))];
    pm2 = [pm2 mean(abs_z(:,2))];
    pm3 = [pm3 mean(abs_z(:,3))];
    pm4 = [pm4 mean(abs_z(:,4))];
    % phase angles of poles
%     pa1 = [pa1 mean(ang_z(:,1))];
%     pa2 = [pa2 mean(ang_z(:,2))];
%     pa3 = [pa3 mean(ang_z(:,3))];
    %pa4 = [pa4 mean(ang_z(:,4))];
    
    
    % spectral power
%     bin = [1 27 53];
%     p1 = [p1 sum(P(1:53))];
%     p2 = [p2  sum(P(54:end))];
%     p3 = [p3 sum(P)];
    
end
PM1 = mean(pm1);
PM2 = mean(pm2);
PM3 = mean(pm3);
PM4 = mean(pm4);

 PA1 = mean(pa1);
 PA2 = mean(pa2);
 PA3 = mean(pa3);
 PA4 = mean(pa4);

% P1 = mean(p1./p3);
% P2 = mean(p2./p3);
% 
% P3 = mean(p1./p2);







% spectral features for systole
ps = [];
p0s = [];
p1s = [];
p2s = [];
p3s = [];
p4s = [];
cens = [];
for i5 = 1:row
      temp_t = A(i5,2):A(i5,3);
      temp_amp = PCG(temp_t);
      
      % spectrum analysis
    
    Ps = abs(fft(temp_amp, 512));
    f1 = 1:256;
        
    Ps = Ps(1:256);
    Ps = Ps.^2;
    % spectral power
    
    if (Ps==0)
        continue;
    end
    
    ps = [ps sum(Ps)];
    p0s = [p0s sum(Ps(1:10))];
    p1s = [p1s sum(Ps(11:20))];
    p2s = [p2s sum(Ps(21:30))];
    p3s = [p3s sum(Ps(31:40))];
    p4s = [p4s sum(Ps(41:50))];    
    cens = [cens sum(Ps*f1)/sum(f1)];
end
 P0s = mean(p0s./ps);
 P1s = mean(p1s./ps);
 P2s = mean(p2s./ps);
 P3s = mean(p3s./ps);
 P4s = mean(p4s./ps);
 meancens = mean(cens);
 stdcens = std(cens);
 
 
 
 
 % spectral features for s1

ps1 = [];
p0s1 = [];
p1s1 = [];
p2s1 = [];
p3s1 = [];
p4s1 = [];
cens1 = [];
for i6 = 1:row
      temp_t = A(i6,1):A(i6,2);
      temp_amp = PCG(temp_t);
      
      % spectrum analysis 
    
    Ps1 = abs(fft(temp_amp,512));
    f1 = 1:256;
    
    Ps1 = Ps1(1:256);
    Ps1 = Ps1.^2;
    % spectral power
    
    if (Ps1==0)
        continue;
    end
    
    ps1 = [ps1 sum(Ps1)];
    p0s1 = [p0s1 sum(Ps1(1:10))];
    p1s1 = [p1s1 sum(Ps1(11:20))];
    p2s1 = [p2s1 sum(Ps1(21:30))];
    p3s1 = [p3s1 sum(Ps1(31:40))];
    p4s1 = [p4s1 sum(Ps1(41:50))];     
    cens1 = [cens1 sum(Ps1*f1)/sum(f1)];
end
 P0s1 = mean(p0s1./ps1);
 P1s1 = mean(p1s1./ps1);
 P2s1 = mean(p2s1./ps1);
 P3s1 = mean(p3s1./ps1);
 P4s1 = mean(p4s1./ps1);
 meancens1 = mean(cens1);
 stdcens1 = std(cens1);
 
 
 
  % spectral features for s2

ps2 = [];
p0s2 = [];
p1s2 = [];
p2s2 = [];
p3s2 = [];
p4s2 = [];
cens2 = [];
for i7 = 1:row
      temp_t = A(i7,3):A(i7,4);
      temp_amp = PCG(temp_t);
      
      % spectrum analysis based on welch algorithm
    
    Ps2 = abs(fft(temp_amp,512));
    f1 = 1:256;
    
    Ps2 = Ps2(1:256);
    Ps2 = Ps2.^2;
    % spectral power
    
    if (Ps2==0)
        continue;
    end
    
    ps2 = [ps2 sum(Ps2)];
    p0s2 = [p0s2 sum(Ps2(1:10))];
    p1s2 = [p1s2 sum(Ps2(11:20))];
    p2s2 = [p2s2 sum(Ps2(21:30))];
    p3s2 = [p3s2 sum(Ps2(31:40))];
    p4s2 = [p4s2 sum(Ps2(41:50))];     
    cens2 = [cens2 sum(Ps2*f1)/sum(f1)];
end
 P0s2 = mean(p0s2./ps2);
 P1s2 = mean(p1s2./ps2);
 P2s2 = mean(p2s2./ps2);
 P3s2 = mean(p3s2./ps2);
 P4s2 = mean(p4s2./ps2);
 meancens2 = mean(cens2);
 stdcens2 = std(cens2);
 
 
   % spectral features for diastole

pd = [];
p0d = [];
p1d = [];
p2d = [];
p3d = [];
p4d = [];
cend = [];
for i8 = 1:1:row-1
      temp_t = A(i8,4):A(i8+1,1);
      temp_amp = PCG(temp_t);
      
      % spectrum analysis
    
    Pd = abs(fft(temp_amp,512));
    f1 = 1:256;
    
    Pd = Pd(1:256);
    Pd = Pd.^2;
    % spectral power
    
    if (Pd==0)
        continue;
    end
    
    pd = [pd sum(Pd)];
    p0d = [p0d sum(Pd(1:10))];
    p1d = [p1d sum(Pd(11:20))];
    p2d = [p2d sum(Pd(21:30))];
    p3d = [p3d sum(Pd(31:40))];
    p4d = [p4d sum(Pd(41:50))];   
    cend = [cend sum(Pd*f1)/sum(f1)];
end
 P0d = mean(p0d./pd);
 P1d = mean(p1d./pd);
 P2d = mean(p2d./pd);
 P3d = mean(p3d./pd);
 P4d = mean(p4d./pd);
 meancend = mean(cend);
 stdcend = std(cend);
 
 
 % spectral features for entire cardiac cycle
 
pcc = [];
p0cc = [];
p1cc = [];
p2cc = [];
p3cc = [];
p4cc = [];
cencc = [];
 
 for i9 = 1:row-1
      temp_t = A(i9,1):A(i9+1,1);
      temp_amp = PCG(temp_t);
      
      % spectrum analysis
    
    Pcc = abs(fft(temp_amp,1024));
    f1 = 1:512;
    
    Pcc = Pcc(1:512);
    Pcc = Pcc.^2;
    % spectral power
    
    if (Pcc==0)
        continue;
    end
    
    pcc = [pcc sum(Pcc)];
    p0cc = [p0cc sum(Pcc(1:20))];
    p1cc = [p1cc sum(Pcc(21:40))];
    p2cc = [p2cc sum(Pcc(41:60))];
    p3cc = [p3cc sum(Pcc(61:80))];
    p4cc = [p4cc sum(Pcc(81:100))];   
    cencc = [cencc sum(Pcc*f1)/sum(f1)];
     
 end
 
 P0cc = mean(p0cc./pcc);
 P1cc = mean(p1cc./pcc);
 P2cc = mean(p2cc./pcc);
 P3cc = mean(p3cc./pcc);
 P4cc = mean(p4cc./pcc);
 meancencc = mean(cencc);
 stdcencc = std(cencc);
 
 


features = [m_RR sd_RR  mean_IntS1 sd_IntS1  mean_IntS2 sd_IntS2  mean_IntSys sd_IntSys  mean_IntDia sd_IntDia m_Ratio_SysRR sd_Ratio_SysRR m_Ratio_DiaRR... 
    sd_Ratio_DiaRR m_Ratio_SysDia sd_Ratio_SysDia m_Amp_SysS1 sd_Amp_SysS1 m_Amp_DiaS2 sd_Amp_DiaS2...
    P0s P1s P2s P3s P0s1 P1s1 P2s1 P3s1 P0s2 P1s2 P2s2 P3s2 P0d P1d P2d P3d PM1 PM2 PM3 PM4 PA1 PA2 PA3 PA4 meancens stdcens meancend stdcend meancens1 stdcens1... 
    meancens2 stdcens2 P0cc P1cc P2cc P3cc meancencc stdcencc];

% if (centroid>0.2)
%     features = [];
end

